#!/bin/sh
#
#   Filename:   cron_xmltv2mei_se.sh
#   Version:    1.0
#
#   Most parts of this script was originally from rt2mei...
#
#   This script has been designed to run on:
#       Linksys NSLU2 with Unslung 6.8/6.10 firmware
#
#   This script grabs the EPG data with tv_grab_se_swedb and converts it with
#   xmltv2mei to a MyStuff.mei.
#   It then transfers the EPG data MyStuff.mei to the Toppy.
#   It does lots of error checking and will go to extraordinay lengths to
#   ensure that all the data is transferred across correctly.
#
#   Usage:  ./cron_xmltv2mei_se.sh [options]
#
#   where options is one or more of
#
#       -q          Run in quiet mode
#
#       -nograb     Omits the downloading of EPG data from the Radio Times
#                   website. It just copies across any existing files.
#
#       -f          suppresses the writing of a Favourites file
#
#       -r          replace ':' in title with '_'
#
#       -new        rejects EPG data if it contains a program that ends before
#                   0000 hrs today
#
#       -mt         accept empty EPG data files
#
#       -i          followed by a path to the directory where the rt2mei.cfg
#                   configuration file is located. If omitted then the
#                   directory path for this script is used.
#
#       -w          followed by the full path to the wget executable forces
#                   wget to be used to grab the EPG data. This is only normally
#                   necessary if you are behind a corporate firewall or
#                   similar. wget is not included in rt2mei.
#
#       -tmp        followed by a path to the temp directory (default is /tmp)
#
#       -ramdisk    Ensures that all files are written to ramdisk (the temp
#                   directory). This is useful if you are running from flash
#                   disk. The default is for MyStuff.mei and My Favourites.mcl
#                   to be written to the directory where this script is.
#       -noramdisk
#
#       -tidyup     Deletes MyStuff.mei and My Favourites.mcl after transfer
#
#   NOTE: All stdout and stderr output is redirected to a tempfile that is
#         deleted at the end of the script.
#         The only output is if the -q switch is not given in which case
#         progess messages are sent to stdout

export PATH=${PATH}:/opt/bin
export PUPPY="/opt/bin/puppy"

days="12"
#xmltv_bin="/opt/bin/tv_grab_se_swedb"
# --quiet is needed for cronjobs
#xmltv_params="--days=7 --quiet"
#outputxml="epg.xml"
xmltv2mei_bin="/opt/bin/xmltv2mei"
# add credits and translate unicode characters (especially Swedish chars)
xmltv2mei_params="-c -t"
lcnfile="/opt/etc/xmltv2mei/lcnfile_se_boxer.conf"


Andrex()
{
# Try 3 times to do the Puppy operation

local i=3
until [ $i -eq 0 ]
do
    txt=`${PUPPY} "$@" 2>$tmpfile`
    if [ $? -eq 0 ] ; then return 0 ; fi
    sleep 2
    ${PUPPY} -qc cancel 2>$tmpfile
    sleep 3
    let i=$i-1
done

return 1
}


ToppyReady()
{
    local wasoff=0
    while ! Andrex -qc dir
    do
        # The Toppy is either Off or in Standby so try again later
        if [ $wasoff -eq 0 ] && [ $quiet -eq 0 ] ; then reverb "The Toppy is not ON" ; fi
        wasoff=1
        sleep 50
    done

    # The Toppy is on
    if [ $wasoff -ne 0 ] ; then
        # Ensure that the Toppy remains on for 2 minutes
        # This is to allow TAPS to load
        local i=12
        until [ $i -eq 0 ]
        do
            if ! Andrex -qc dir ; then
                # The Toppy has gone again so we have to start over
                if [ $quiet -eq 0 ] ; then reverb "The Toppy has just gone OFF or into STANDBY" ; fi
                return 1
            fi
            sleep 8
            let i=$i-1
        done
    fi

    # The Toppy is fully awake
    return 0
}


Transfer()
{
    # Wait for the Toppy to be fully awake
    ready=0 ; until [ $ready -eq 1 ]
    do
        ToppyReady
        if [ $? -eq 0 ] ; then
            ready=1         # The Toppy is ready
        else
            reverb "Toppy not ready"
            sleep 60        # try again later
        fi
    done

    # Check that MyStuff is not reading the MEI data
    ok=0 ; until [ $ok -eq 1 ]
    do
        if Andrex -qc dir ProgramFiles ; then
            if echo $txt | grep -q $readlockfile ; then
                sleep 10    # Lockfile present so try again later
            else
                ok=1        # Lockfile not present so continue
            fi
        else
            return 1        # Puppy failed
        fi
    done

    # We are free to copy files to the Toppy

    # Tell MyStuff not to read the files while we are writing them
    if [ $quiet -eq 0 ] ; then reverb 'Locking MyStuff' ; fi
    if [ -e "$writepath/$writelockfile" ] ; then
        if ! Andrex -qc put $writepath/$writelockfile $toppydir$writelockfile ; then
            # We failed to write the lockfile to the Toppy
            if [ $quiet -eq 0 ] ; then reverb "ERROR: Failed to copy $writelockfile to the Toppy" ; fi
            return 1            # Puppy failed
        fi
    else
        if [ $quiet -eq 0 ] ; then reverb "ERROR: Cannot find $writepath/$writelockfile to copy to the Toppy" ; fi
        exit 1
    fi

    # Copy the new MEI file to the Toppy
    if [ -e "$writepath/$meifile" ] ; then
        if [ $quiet -eq 0 ] ; then reverb "Copying $meifile to the Toppy" ; fi
        if ! Andrex -qc put "$writepath/$meifile" $toppydir$meifile ; then
            # We failed to write MyStuff.mei to the Toppy
            if [ $quiet -eq 0 ] ; then reverb "ERROR: Failed to copy $meifile to the Toppy" ; fi
            return 1            # Puppy failed
        fi
    else
        if [ $quiet -eq 0 ] ; then reverb "ERROR: Cannot find $writepath/$meifile to copy to the Toppy" ; fi
        exit 1
    fi

    # If there is a Favourites file then copy that too
    if [ -e "$writepath/$favsfile" ] ; then
        # There is a favourites file to copy

        if [ $quiet -eq 0 ] ; then reverb "Copying $favsfile to the Toppy" ; fi
        if ! Andrex -qc put "$writepath/$favsfile" "$toppydir$favsfile" ; then
            # We failed to copy the Favourites file to the Toppy
            if [ $quiet -eq 0 ] ; then reverb "WARNING: Failed to copy $favsfile to the Toppy" ; fi
            return 1        # Puppy failed
        fi
    fi

    # Tell MyStuff that new data is present
    if [ $quiet -eq 0 ] ; then reverb 'Flagging MyStuff' ; fi
    if [ -e "$writepath/$flagfile" ] ; then
        if ! Andrex -qc put $writepath/$flagfile $toppydir$flagfile ; then
            # We failed to write the flagfile to the Toppy
            if [ $quiet -eq 0 ] ; then reverb "ERROR: Failed to copy $flagfile to the Toppy" ; fi
            return 1            # Puppy failed
        fi
    else
        if [ $quiet -eq 0 ] ; then reverb "ERROR: Cannot find $writepath/$flagfile to copy to the Toppy" ; fi
        exit 1
    fi

    # Release the lock
    if [ $quiet -eq 0 ] ; then reverb 'Unlocking MyStuff' ; fi
    if ! Andrex -qc delete $toppydir$writelockfile ; then
        # We failed to delete the lockfile from the Toppy
        if [ $quiet -eq 0 ] ; then reverb "ERROR: Failed to delete $writelockfile from the Toppy" ; fi
        return 1            # Puppy failed
    fi

    return 0
}


reverb()
{
    dt=`date +"%d/%m/%y %X"`
    echo "$dt: $@"
}


# *****************************************************************************
# **************************                        ***************************
# **************************    Main entry point    ***************************
# **************************                        ***************************
# *****************************************************************************

# Initialise some variables
tmp='/tmp'
#pathname=`dirname $0`
pathname=`echo ${0%/*}`
writepath=$pathname
myname=`basename $0`
readlockfile='MyStuff.mei.reading'
writelockfile='MyStuff.mei.writing'
flagfile='MyStuff.mei.written'
meifile='MyStuff.mei'
favsfile='My Favourites.mcl'
toppydir='ProgramFiles\'
bakext='.bak'


# Kill any old bits that are running
pids=`pidof $myname`
for pid in $pids ; do if [ $pid != $$ ] ; then kill $pid ; fi ; done
pids=`pidof puppy`
if [ "$pids" != "" ] ; then kill $pids ; fi

# Process command line arguments
quiet=0
grab=1
ramdisk=1
tidyup=0
params=''

while [ "$1" ]
do
    case $1 in
    "-q")
        params="$params $1"
        quiet=1 ;;
    "-nograb")
        grab=0 ;;
    "-f" | "-r" | "-new" | "-mt")
        params="$params $1" ;;
    "-i" | "-w")
        params="$params $1 $2"
        shift ;;
    "-o" | "-t")
        echo "ERROR: Option $1 is explicitly disallowed"
        exit 1 ;;
    "-tmp")
        if [ "$2" != "" ] ; then
            tmp=`echo "$2" | sed 's|/$||'`
            shift
        fi ;;
    "-ramdisk")
        ramdisk=1 ;;
    "-noramdisk")
        ramdisk=0 ;;
    "-tidyup")
        tidyup=1 ;;
    *)
        cat <<USAGE
#   Filename:   cron_xmltv2mei_se.sh
#   Version:    1.0
#
#   This script has been designed to run on:
#       Linksys NSLU2 with Unslung 6.8/6.10 firmware
#
#   This script grabs the EPG data with tv_grab_se_swedb and converts it with
#   xmltv2mei to a MyStuff.mei.
#   It then transfers the EPG data MyStuff.mei to the Toppy.
#   It does lots of error checking and will go to extraordinay lengths to
#   ensure that all the data is transferred across correctly.
#
#   Usage:  ./cron_xmltv2mei_se.sh [options]
#
#   where options is one or more of
#
#       -q          Run in quiet mode
#
#       -nograb     Omits the downloading of EPG data from the Radio Times
#                   website. It just copies across any existing files.
#
#       -f          suppresses the writing of a Favourites file
#
#       -r          replace ':' in title with '_'
#
#       -new        rejects EPG data if it contains a program that ends before
#                   0000 hrs today
#
#       -mt         accept empty EPG data files
#
#       -i          followed by a path to the directory where the rt2mei.cfg
#                   configuration file is located. If omitted then the
#                   directory path for this script is used.
#
#       -w          followed by the full path to the wget executable forces
#                   wget to be used to grab the EPG data. This is only normally
#                   necessary if you are behind a corporate firewall or
#                   similar. wget is not included in rt2mei.
#
#       -tmp        followed by a path to the temp directory (default is /tmp)
#
#       -ramdisk    Ensures that all files are written to ramdisk (the temp
#                   directory). This is useful if you are running from flash
#                   disk. The default is for MyStuff.mei and My Favourites.mcl
#                   to be written to the directory where this script is.
#       -noramdisk
#
#       -tidyup     Deletes MyStuff.mei and My Favourites.mcl after transfer
#
#   NOTE: All stdout and stderr output is redirected to a tempfile that is
#         deleted at the end of the script.
#         The only output is if the -q switch is not given in which case
#         progess messages are sent to stdout
USAGE
        exit 1 ;;
    esac
    shift
done

tmpfile="$tmp/xmltv2mei_$$.tmp"
if [ $ramdisk -eq 1 ] ; then
    writepath=$tmp
    params="$params -o $tmp -t $tmp"
else
    writepath=$pathname
fi

if [ "$params" != "" ] ; then params="-- $params" ; fi

if [ $quiet -eq 0 ] ; then reverb 'Job started' ; fi


# Make sure that these files exist
echo 'Just hold on there a moment, BobD' >$writepath/$writelockfile
echo 'Go get it, BobD' >$writepath/$flagfile

# Now try to grab the EPG data
if [ $grab -eq 1 ] ; then
    # Backup the old MEI file
    if [ -e "$writepath/$meifile" ] ; then
        # A previous MEI file exists so make a backup
        if [ -e "$writepath/$meifile$bakext" ] ; then
            # There is an old backup so delete it
            rm -f "$writepath/$meifile$bakext"
        fi
    
        mv "$writepath/$meifile" "$writepath/$meifile$bakext"
    fi

    # Grab the EPG data with xmltv
#    rm -f "$writepath/$outputxml"
#    $xmltv_bin $xmltv_params --output=$writepath/$outputxml
#    retval=$?
#    if [ $retval -eq 0 ] ; then
#        reverb "EPG grabbed" >>$writepath/epg.log
#    else
#	rm -f "$writepath/$outputxml"
#        reverb "Error found by xmltv" >>$writepath/epg.log
#    fi

#    if [ ! -f "$writepath/$outputxml" ] ; then
#        reverb "xmltv failed to download epg-data" >>$writepath/epg.log
#    fi

    # Convert to mei-file with xmltv2mei
#    $xmltv2mei_bin $xmltv2mei_params -i "$writepath/$outputxml" -l "$lcnfile" -o "$writepath/$meifile" 2>&1 >/dev/null </dev/null
    $xmltv2mei_bin $xmltv2mei_params -x $days -l "$lcnfile" -o "$writepath/$meifile" 2>&1 >/dev/null </dev/null
    retval=$?

    if [ $retval -eq 0 ] ; then
        reverb "EPG grabbed" >>$writepath/epg.log
    else
        rm -f "$writepath/$meifile"
        reverb "xmltv2mei failed to generate mei-file" >>$writepath/epg.log
    fi
else
    if [ $quiet -eq 0 ] ; then reverb 'Radio Times data will not be grabbed' ; fi
fi

if [ -e "$writepath/$meifile" ] ; then
    # MEI file exists so transfer it to the Toppy
    # Keep trying until we get there

    completed=0
    until [ $completed -eq 1 ]
    do
        Transfer
        STATUS=$?
        if [ $STATUS -eq 0 ] ; then
         SZ="`${PUPPY} -c dir ProgramFiles | grep MyStuff | grep -v written | awk '{print $2;}'`"
         ORGSZ="`/bin/ls -l "$writepath/$meifile" | awk '{print $5}'`"
         if [ "$SZ" = "$ORGSZ" ]; then
           completed=1
           reverb "Uploaded file-size is ok $SZ == $ORGSZ"
         else
           reverb "Uploaded file-size differ $SZ != $ORGSZ"
         fi
        fi
    done

    if [ $quiet -eq 0 ] ; then reverb 'Job completed successfully' ; fi
else
    if [ $quiet -eq 0 ] ; then reverb 'No MEI file found to transfer' ; fi
fi

if [ $tidyup -eq 1 ] ; then
    rm -f "$writepath/$meifile"
    rm -f "$writepath/$favsfile"
    rm -f $writepath/$writelockfile
    rm -f $writepath/$flagfile
fi

rm -f $tmpfile
exit 0

#EOF